/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Document;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.ImportedDocuments;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.SchemaReader;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaElementImpl;
import org.ow2.easywsdl.schema.api.extensions.NamespaceMapperImpl;
import org.ow2.easywsdl.schema.api.extensions.SchemaLocatorImpl;
import org.ow2.easywsdl.schema.impl.SchemaImpl;
import org.ow2.easywsdl.schema.impl.SchemaReaderImpl;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLImportException;
import org.ow2.easywsdl.wsdl.api.WSDLReader.FeatureConstants;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLElementImpl;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;
import org.ow2.easywsdl.wsdl.org.w3.ns.wsdl.DescriptionType;
import org.ow2.easywsdl.wsdl.org.xmlsoap.schemas.wsdl.TDefinitions;

/**
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class DocumentImpl extends AbstractWSDLElementImpl<org.ow2.easywsdl.extensions.complexwsdl.Document> implements Document {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(DocumentImpl.class.getName());

	private Description imptDesc = null;

	private Schema imptSchema = null;
	
	private SchemaReader schemaReader = createSchemaReader();
	

    private static SchemaReader createSchemaReader() {
        try {
            return new SchemaReaderImpl();
        } catch (SchemaException e) {
            throw new RuntimeException(e);
        }
    }	


    public DocumentImpl(final org.ow2.easywsdl.extensions.complexwsdl.Document doc, final ImportedDocuments docs) throws WSDL4ComplexWsdlException {
		super(doc, (AbstractWSDLElementImpl) docs);
		try {
			if (doc.getDefinition() != null) {
				Map<FeatureConstants, Object> features = new HashMap<FeatureConstants, Object>();
				imptDesc = new org.ow2.easywsdl.wsdl.impl.wsdl11.DescriptionImpl(new URI("."), doc.getDefinition(), new NamespaceMapperImpl(), new SchemaLocatorImpl(), features, (Map<URI, AbsItfDescription>) docs.getAllDescriptions(), (Map<URI, AbsItfSchema>) docs.getAllSchemas(), null);
			} else if (doc.getDescription() != null) {
				Map<FeatureConstants, Object> features = new HashMap<FeatureConstants, Object>();
				imptDesc = new org.ow2.easywsdl.wsdl.impl.wsdl20.DescriptionImpl(new URI("."), doc.getDescription(), new NamespaceMapperImpl(), new SchemaLocatorImpl(), features, (Map<URI, AbsItfDescription>) docs.getAllDescriptions(), (Map<URI, AbsItfSchema>) docs.getAllSchemas(), null);
			} else if (doc.getSchema() != null) {
				Map<org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants, Object> features = new HashMap<org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants, Object>();
				imptSchema = new SchemaImpl(new URI("."), doc.getSchema(), new NamespaceMapperImpl(), new SchemaLocatorImpl(), features, (Map<URI, AbsItfSchema>) docs.getAllSchemas(), (SchemaReaderImpl) schemaReader);
				((SchemaImpl) imptSchema).initialize();
			}
		} catch (WSDLException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (WSDLImportException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (URISyntaxException e) {
			throw new WSDL4ComplexWsdlException(e);
		} catch (SchemaException e) {
			throw new WSDL4ComplexWsdlException(e);
		}
	}

	public URI getLocation() {
		URI res = null;
		if(this.model.getLocation() != null) {
			res = URI.create(this.model.getLocation());
		}
		return res;
	}

	public void setLocation(URI location) {
		this.model.setLocation(location.toString());
	}

	public Description getImportedDescription() {
		return this.imptDesc;
	}

	public Schema getImportedSchema() {
		return this.imptSchema;
	}

	public void setImportedDescription(Description desc) {
		this.imptDesc = desc;
		if (((AbstractWSDLElementImpl) desc).getModel() instanceof TDefinitions) {
			this.model.setDefinition((TDefinitions) ((AbstractWSDLElementImpl) desc).getModel());
		} else if (((AbstractWSDLElementImpl) desc).getModel() instanceof DescriptionType) {
			this.model.setDescription((DescriptionType) ((AbstractWSDLElementImpl) desc).getModel());
		}
	}

	public void setImportedSchema(Schema schema) {
		this.imptSchema = schema;
		this.model.setSchema((org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema) ((AbstractSchemaElementImpl) schema).getModel());
	}

	public URI getOriginalLocation() {
		return URI.create(this.model.getOriginalLocation());
	}

	public void setOriginalLocation(URI originalLocation) {
		this.model.setOriginalLocation(originalLocation.toString());
	}
}
