/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.wsdl4complexwsdl.impl;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Document;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.ImportedDocuments;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.abstractElmt.AbstractWSDLElementImpl;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;

/** 
* @author Nicolas Salatge - EBM WebSourcing
*/
public class ImportedDocumentsImpl extends AbstractWSDLElementImpl<org.ow2.easywsdl.extensions.complexwsdl.ImportedDocuments> implements ImportedDocuments {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private List<Document> documents = new ArrayList<Document>();
	
	private Description description = null;
	
    public ImportedDocumentsImpl(final org.ow2.easywsdl.extensions.complexwsdl.ImportedDocuments documents, final Description desc) throws WSDL4ComplexWsdlException {
        super(documents, (AbstractWSDLElementImpl) desc);
        this.description = desc;

     
        for(org.ow2.easywsdl.extensions.complexwsdl.Document doc: this.model.getDocument()) {
        	this.documents.add(new DocumentImpl(doc, this));
        }
    }

	/**
	 * @return the desc
	 */
	public Description getDescription() {
		return description;
	}

	public void addDocument(Document doc) {
		this.model.getDocument().add((org.ow2.easywsdl.extensions.complexwsdl.Document) ((AbstractWSDLElementImpl)doc).getModel());
		this.documents.add(doc);
	}

	public List<Document> getDocuments() {
		return this.documents;
	}

	public void addAllDocuments(Collection<Document> docs) {
		for(Document doc: docs) {
			this.addDocument(doc);
		}
	}
	
	public Map<URI, AbsItfDescription> getAllDescriptions() {
		Map<URI, AbsItfDescription> res = new HashMap<URI, AbsItfDescription>();
		for(Document doc: this.documents) {
			if(doc.getImportedDescription() != null) {
				res.put(doc.getImportedDescription().getDocumentBaseURI(), doc.getImportedDescription());
			}
		}
		return res;
	}
	
	public Map<URI, AbsItfSchema> getAllSchemas() {
		Map<URI, AbsItfSchema> res = new HashMap<URI, AbsItfSchema>();
		for(Document doc: this.documents) {
			if(doc.getImportedSchema() != null) {
				res.put(doc.getImportedSchema().getDocumentURI(), doc.getImportedSchema());
			}
		}
		return res;
	}
	
}
